@@ -6,71 +6,62 @@
 
 #include "edit.h"
 #ifdef UNIX
-#ifndef DJGPP
 
-#include <termio.h>
+#include <termios.h>
+#include <sys/ioctl.h>
 #include <fcntl.h>
 
-/*
-#undef NL1
-#undef CR1
-#undef CR2
-#undef CR3
-#undef TAB1
-#undef BS1
-#undef FF1
-#undef B50
-#undef B75
-#undef B110
-#undef B134
-#undef B150
-#undef B200
-#undef B300
-#undef B600
-#undef B1200
-#undef B1800
-#undef B2400
-#undef B4800
-#undef B9600
-#undef EXTA
-#undef EXTB
-#undef HUPCL
-#undef ECHO
-#undef TIOCHPCL
-#undef TIOCEXCL
-#undef TIOCNXCL
-
-#undef O_SYNC
-#define O_SYNC 020
-
-#ifndef TERMCAP
-#include<curses.h>
-#ifndef DJGPP
-#include<term.h>
-#endif
-#endif
-*/
 #include<signal.h>
 #define KEYFN 42
 #define NSPCHR 13
 
-int MAXSLNS = 24;
-int MAXSCOL = 80;
+#ifndef XWPE_DLL
+#define WpeDllInit WpeTermInit
+#endif
+
+/*    we_term.c    */
+char *init_key(char *key);
+char *init_kkey(char *key);
+int init_cursor(void);
+int e_begscr(void);
+void e_endwin(void);
+int fk_t_cursor(int x);
+int fk_t_putchar(int c);
+int fk_attrset(int a);
+int e_t_refresh(void);
+int e_t_sys_ini(void);
+int e_t_sys_end(void);
+int e_t_getch(void);
+int e_find_key(int c, int j, int sw);
+void e_exitm(char *s, int n);
+int fk_t_locate(int x, int y);
+int fk_t_mouse(int *g);
+int e_t_initscr(void);
+int e_t_kbhit(void);
+int e_t_d_switch_out(int sw);
+int e_t_switch_screen(int sw);
+int e_t_deb_out(FENSTER *f);
+int e_s_sys_end();
+int e_s_sys_ini();
+
+extern int MAXSLNS;
+extern int MAXSCOL;
+
 
 char *key_f[KEYFN], *key_key;
 char *cur_rc, *cur_vs, *cur_nvs, *cur_vvs, cur_attr;
-char *att_no, *att_so, *att_ul, *att_rv, *att_bl, *att_dm, *att_bo;
+char *att_so, *att_ul, *att_rv, *att_bl, *att_dm, *att_bo;
 char *ratt_no, *ratt_so, *ratt_ul, *ratt_rv, *ratt_bl, *ratt_dm, *ratt_bo;
 char *beg_scr, *swt_scr, *sav_cur, *res_cur; 
-char *altschirm;
+extern char *altschirm;
+extern char *att_no;
 char *col_fg, *col_bg, *spc_st, *spc_in, *spc_bg, *spc_nd;
 
-int col_num = 0;
+extern int col_num;
 char *sp_chr[NSPCHR];
 
-int cur_x = -1, cur_y = -1;
-struct termio otermio, ntermio, ttermio;
-/*  TERMINAL *save_term;   */
+extern int cur_x, cur_y;
+extern struct termios otermio, ntermio, ttermio;
 #ifdef TERMCAP
 char area[315];
 char *ap = area;
@@ -79,82 +70,192 @@ char *tc_screen;
 char *tgetstr();
 int tgetnum();
 char *tgoto();
-/*  short int colums, lines;   */
 #define tigetstr(k) tgetstr(k, &ap)
 #else
-char *tigetstr();
-int tigetnum();
-char *tparm(/* ??? */);
-int endwin();
-int noecho();
+/* Because term.h defines "buttons" it messes up we_gpm.c if in edit.h */
+#include <term.h>
+
+/* AIX requires that tparm has 10 arguments. */
+#define tparm1(a,b) tparm((a), (b), 0, 0, 0, 0, 0, 0, 0, 0)
+#define tparm2(a,b,c) tparm((a), (b), (c), 0, 0, 0, 0, 0, 0, 0)
 #endif
-int def_shell_mode();
-int initscr(/* ??? */);
-int tputs();
-int putp();
-int setupterm();
-int e_switch_screen();
 
 extern int e_term_def;
 
-char *init_key(key)
-     char *key;
+int WpeDllInit(int *argc, char **argv)
 {
-   char *tmp, *keystr;
-   tmp = (char *) tigetstr(key);
-   if(tmp == NULL || tmp == ((char *) -1)) return(NULL);
-   else
-   {  keystr = MALLOC(strlen(tmp)+1);
-      strcpy(keystr, tmp);
-   }
-   return(keystr);
+ fk_u_cursor = fk_t_cursor;
+ fk_u_locate = fk_t_locate;
+ e_u_d_switch_out = e_t_d_switch_out;
+ e_u_switch_screen = e_t_switch_screen;
+ e_u_deb_out = e_t_deb_out;
+ e_u_kbhit = e_t_kbhit;
+ e_u_s_sys_end = e_s_sys_end;
+ e_u_s_sys_ini = e_s_sys_ini;
+ e_u_refresh = e_t_refresh;
+ e_u_getch = e_t_getch;
+ e_u_sys_ini = e_t_sys_ini;
+ e_u_sys_end = e_t_sys_end;
+ fk_u_putchar = fk_t_putchar;
+#ifdef GPM
+ if (WpeGpmMouseInit() == 0)
+ {
+  fk_mouse = WpeGpmMouse;
+ }
+ else
+#endif
+  fk_mouse = fk_t_mouse;
+ WpeMouseChangeShape = (void (*)(WpeMouseShape))WpeNullFunction;
+ WpeMouseRestoreShape = WpeNullFunction;
+ WpeDisplayEnd = e_endwin;
+#ifdef __linux__
+ u_bioskey = WpeLinuxBioskey;
+#else
+ u_bioskey = WpeZeroFunction;
+#endif
+ e_t_initscr();
+ if (col_num > 0)
+ {
+  e_pr_u_col_kasten = e_pr_x_col_kasten;
+  e_frb_u_menue = e_frb_x_menue;
+  e_s_u_clr = e_s_x_clr;
+  e_n_u_clr = e_n_x_clr;
+ }
+ else
+ {
+  e_pr_u_col_kasten = e_pr_t_col_kasten;
+  e_frb_u_menue = e_frb_t_menue;
+  e_s_u_clr = e_s_t_clr;
+  e_n_u_clr = e_n_t_clr;
+ }
+ MCI = '+';
+ MCA = '0';
+ RD1 = '\01';
+ RD2 = '\02';
+ RD3 = '\03';
+ RD4 = '\04';
+ RD5 = '\05';
+ RD6 = '\06';
+ RE1 = '\01';
+ RE2 = '\02';
+ RE3 = '\03';
+ RE4 = '\04';
+ RE5 = '\05';
+ RE6 = '\06';
+ WBT = '\13';
+ ctree[0] = "\11\12\07";   /*  07 -> 30  */
+ ctree[1] = "\11\12\12";	/*  11 -> 16  */
+ ctree[2] = "\11\07\12";   /*  12 -> 22  */
+ ctree[3] = "\10\12\12";   /*  10 -> 25  */
+ ctree[4] = "\11\12\12";
+/*
+ RD1 = '+';
+ RD2 = '+';
+ RD3 = '+';
+ RD4 = '+';
+ RD5 = '-';
+ RD6 = '|';
+*//*
+ RE1 = '.';
+ RE2 = '.';
+ RE3 = '.';
+ RE4 = '.';
+ RE5 = '.';
+ RE6 = ':';
+ WBT = '#';
+ ctree[0] = "|__";
+ ctree[1] = "|__";
+ ctree[2] = "|__";
+ ctree[3] = "|__";
+ ctree[4] = "|__";
+*/
+ return 0;
 }
 
-char *init_kkey(key)
-     char *key;
+char *init_key(char *key)
 {
-   char *tmp;
-   int i;
-   tmp = init_key(key);
-   if(tmp == NULL) return(NULL);
-   if(!key_key)
-   {  key_key = MALLOC(2); key_key[0] = tmp[1];
-      key_key[1] = '\0';  return(tmp);  }
-   else
-   {  for(i = 0; key_key[i] != '\0'; i++)
-      if(key_key[i] == tmp[1]) return(tmp);
-      key_key = REALLOC(key_key, i + 2);
-      key_key[i] = tmp[1];
-      key_key[i + 1] = '\0';
-   }
-   return(tmp);
+ char *tmp, *keystr;
+
+ tmp = (char *) tigetstr(key);
+ if (tmp == NULL || tmp == ((char *) -1))
+  return(NULL);
+ else
+ {
+  keystr = MALLOC(strlen(tmp)+1);
+  strcpy(keystr, tmp);
+ }
+ return(keystr);
 }
 
-char *init_spchr(c)
-     char c;
+char *init_kkey(char *key)
 {
-   int i;
-   char *pt = NULL;
-   if(!spc_st || !spc_bg || !spc_nd) return(NULL);
-   for(i = 0; spc_st[i] && spc_st[i+1] && spc_st[i] != c; i+=2);
-   if(spc_st[i] && spc_st[i+1])
-   {  pt = MALLOC((strlen(spc_bg)+strlen(spc_nd)+2)*sizeof(char));
-      if(pt) sprintf(pt, "%s%c%s", spc_bg, spc_st[i+1], spc_nd);
-   }
-   return(pt);
+ char *tmp;
+ int i;
+
+ tmp = init_key(key);
+ if (tmp == NULL)
+  return(NULL);
+ if (!key_key)
+ {
+  key_key = MALLOC(2);
+  key_key[0] = tmp[1];
+  key_key[1] = '\0';
+  return(tmp);
+ }
+ else
+ {
+  for (i = 0; key_key[i] != '\0'; i++)
+   if (key_key[i] == tmp[1])
+    return(tmp);
+  key_key = REALLOC(key_key, i + 2);
+  key_key[i] = tmp[1];
+  key_key[i + 1] = '\0';
+ }
+ return(tmp);
+}
+
+char *init_spchr(char c)
+{
+ int i;
+ char *pt = NULL;
+
+ if (!spc_st || !spc_bg || !spc_nd)
+  return(NULL);
+ for (i = 0; spc_st[i] && spc_st[i+1] && spc_st[i] != c; i+=2)
+  ;
+ if (spc_st[i] && spc_st[i+1])
+ {
+  pt = MALLOC((strlen(spc_bg)+strlen(spc_nd)+2)*sizeof(char));
+  if(pt)
+   sprintf(pt, "%s%c%s", spc_bg, spc_st[i+1], spc_nd);
+ }
+ return(pt);
 }
 
 int init_cursor()
 {
-/*   int i;     */
 #ifndef TERMCAP
-   if(!(cur_rc = init_key("cup"))) return(-1);
-   if((col_fg = init_key("setf")) && (col_bg = init_key("setb")))
-   {  if((col_num = tigetnum("colors")) < 0) col_num = 8;  }
-   else
-   {  if(col_fg) {  free(col_fg);  col_fg = NULL;   }
-      if(col_bg) {  free(col_bg);  col_bg = NULL;   }
+   WINDOW * stdscr;
+
+   if (!(cur_rc = init_key("cup")))
+    return(-1);
+   if ((col_fg = init_key("setaf")) && (col_bg = init_key("setab")))
+   { /* terminfo 10.2.x color code */
+    if ((col_num = tigetnum("colors")) < 0) col_num = 8;
    }
+   else if ((col_fg = init_key("setf")) && (col_bg = init_key("setb")))
+   { /* Older terminfo color code */
+    if((col_num = tigetnum("colors")) < 0) col_num = 8;
+   }
+   else
+   { /* No colors */
+    if(col_fg) {  free(col_fg);  col_fg = NULL;   }
+    if(col_bg) {  free(col_bg);  col_bg = NULL;   }
+   }
+
+   if ((stdscr=initscr())==(WINDOW *)ERR) exit(27);
+
+   start_color();
    spc_st = init_key("acsc");
    spc_in = init_key("enacs");
    spc_bg = init_key("smacs");
@@ -204,7 +305,6 @@ int init_cursor()
    {  key_f[14] = init_kkey("kich1");
       key_f[15] = init_kkey("khome");
       key_f[16] = init_kkey("kpp");
-/*   	key_f[17] = init_kkey("kdch1");      */
       if(!(key_f[17] = init_kkey("kdch1"))) key_f[17] = "\177";
       key_f[18] = init_kkey("kend");
       key_f[19] = init_kkey("knp");
@@ -322,16 +422,11 @@ int init_cursor()
    if(!(sp_chr[12] = init_spchr('a'))) sp_chr[12] = " ";
    return(0);
 }
-/*
-void e_ini_schirm()
-{
-    extern struct EXT h_error;
-    e_initscr();
-}
-*/
+
 int e_t_initscr()
 {  int ret;
-   ret = ioctl(1, TCGETA, &otermio);     /* save old settings */
+   ret = tcgetattr(1, &otermio); /* save old settings */
+/*   ret = ioctl(1, TCGETA, &otermio);     / save old settings */
 /*
     if(ret)
     {	printf("Error in Terminal Initialisation Code: %d\n", ret);
@@ -341,21 +436,13 @@ int e_t_initscr()
 		otermio.c_lflag, otermio.c_line, otermio.c_cc[0], otermio.c_cc[1], 
 		otermio.c_cc[2], otermio.c_cc[3], otermio.c_cc[4], otermio.c_cc[5], 
 		otermio.c_cc[6], otermio.c_cc[7]);
-	exit(1);
+        WpeExit(1);
     }
 */
-/*
-#ifndef TERMCAP
-   def_shell_mode();
-   initscr();
-#endif
-*/
-/*    delwin(stdscr);    */
-/*    save_term = (TERMINAL *) MALLOC(sizeof(TERMINAL));   */
    e_begscr();
    schirm = MALLOC(2 * MAXSCOL * MAXSLNS);
    altschirm = MALLOC(2 * MAXSCOL * MAXSLNS);
-#if defined(XWINDOW) && defined(NEWSTYLE)
+#if !defined(NO_XWINDOWS) && defined(NEWSTYLE)
    extbyte = MALLOC(MAXSCOL * MAXSLNS);
 #endif
    e_abs_refr();
@@ -363,16 +450,18 @@ int e_t_initscr()
    {  printf("Terminal Not in the right mode\n");
       e_exit(1);
    }
-   ioctl(0, TCGETA, &ntermio);
+   tcgetattr(0, &ntermio);
+/*   ioctl(0, TCGETA, &ntermio);*/
    ntermio.c_iflag = 0;            /* setup new settings */
    ntermio.c_oflag = 0;
    ntermio.c_lflag = 0;
    ntermio.c_cc[VMIN] = 1;
    ntermio.c_cc[VTIME] = 0;
-#ifndef NOVSWTCH
+#ifdef VSWTCH
    ntermio.c_cc[VSWTCH] = 0;
 #endif
-   ioctl(0, TCSETA, &ntermio);
+   tcsetattr(0, TCSADRAIN, &ntermio);
+/*   ioctl(0, TCSETA, &ntermio);*/
    if(spc_in) e_putp(spc_in);
    return(0);
 }
@@ -381,89 +470,72 @@ int svflgs, kbdflgs;
 
 int e_begscr()
 {
-   int cols, lns;
-   kbdflgs = fcntl( 0, F_GETFL, 0 );
+ int cols, lns;
+
+ kbdflgs = fcntl( 0, F_GETFL, 0 );
 #ifndef TERMCAP
-/*   noecho();    */
-   setupterm((char *)0, 1, (int *)0);
-   if((lns = tigetnum("lines")) > 0) MAXSLNS = lns;
-   if((cols = tigetnum("cols")) > 0) MAXSCOL = cols;
+ setupterm((char *)0, 1, (int *)0);
+ if ((lns = tigetnum("lines")) > 0)
+  MAXSLNS = lns;
+ if ((cols = tigetnum("cols")) > 0)
+  MAXSCOL = cols;
 #else
-   if ((tc_screen = getenv("TERM")) == NULL)
-   e_exitm("Environment variable TERM not defined!", 1);
-   if ((tgetent(tcbuf, tc_screen)) != 1)
-   e_exitm("Unknown terminal type!", 1);
-   if((lns = tgetnum("li")) > 0) MAXSLNS = lns;
-   if((cols = tgetnum("co")) > 0) MAXSCOL = cols;
-/*    
-    if ((lines=(short)tgetnum("li")) == -1)
-		e_exitm("termcap entry incomplete (lines)", 1);
-    if ((colums=(short)tgetnum("co")) == -1)
-		e_exitm("termcap entry incomplete (colums)", 1);
-*/
+ if ((tc_screen = getenv("TERM")) == NULL)
+  e_exitm("Environment variable TERM not defined!", 1);
+ if ((tgetent(tcbuf, tc_screen)) != 1)
+  e_exitm("Unknown terminal type!", 1);
+ if ((lns = tgetnum("li")) > 0)
+  MAXSLNS = lns;
+ if ((cols = tgetnum("co")) > 0)
+  MAXSCOL = cols;
 #endif
-/*    *save_term = *cur_term;   */
-   
-   signal(SIGSEGV, e_exit);
-   
-   return(0);
+ return(0);
 }
 
 #define fk_putp(p) ( p ? e_putp(p) : e_putp(att_no) )
 
-int e_endwin()
+void e_endwin()
 {
-/*
-#ifndef TERMCAP
-   endwin();
-#endif
-*/
-   fk_putp(ratt_bo);
-   ioctl(0, TCSETA, &otermio);
-   return(0);
+ fk_putp(ratt_bo);
+ tcsetattr(0, TCSADRAIN, &otermio);
+/* ioctl(0, TCSETA, &otermio);*/
 }
 
-int fk_t_cursor(x)
-     int x;
+int fk_t_cursor(int x)
 {
-   return(x);
-/*    if(x == 0) putp(cur_nvs);
-    else putp(cur_vvs);    */
+ return(x);
 }
 
-int fk_t_putchar(c)
-     char c;
+int fk_t_putchar(int c)
 {
-   return(fputc(c, stdout));
+ return(fputc(c, stdout));
 }
 
-int fk_attrset(a)
-     int a;
+int fk_attrset(int a)
 {
-   if(cur_attr == a) return(0);
-   switch(cur_attr)
-   {  case 0:  fk_putp(ratt_no);  break;
-      case 1:  fk_putp(ratt_so);  break;
-      case 2:  fk_putp(ratt_ul);  break;
-      case 4:  fk_putp(ratt_rv);  break;
-      case 8:  fk_putp(ratt_bl);  break;
-      case 16:  fk_putp(ratt_dm);  break;
-      case 32:  fk_putp(ratt_bo);  break;
-   }
-   switch(a)
-   {  case 0:  fk_putp(att_no);  break;
-      case 1:  fk_putp(att_so);  break;
-      case 2:  fk_putp(att_ul);  break;
-      case 4:  fk_putp(att_rv);  break;
-      case 8:  fk_putp(att_bl);  break;
-      case 16:  fk_putp(att_dm);  break;
-      case 32:  fk_putp(att_bo);  break;
-   }
-   return(cur_attr = a);
+ if(cur_attr == a) return(0);
+ switch(cur_attr)
+ {  case 0:  fk_putp(ratt_no);  break;
+    case 1:  fk_putp(ratt_so);  break;
+    case 2:  fk_putp(ratt_ul);  break;
+    case 4:  fk_putp(ratt_rv);  break;
+    case 8:  fk_putp(ratt_bl);  break;
+    case 16:  fk_putp(ratt_dm);  break;
+    case 32:  fk_putp(ratt_bo);  break;
+ }
+ switch(a)
+ {  case 0:  fk_putp(att_no);  break;
+    case 1:  fk_putp(att_so);  break;
+    case 2:  fk_putp(att_ul);  break;
+    case 4:  fk_putp(att_rv);  break;
+    case 8:  fk_putp(att_bl);  break;
+    case 16:  fk_putp(att_dm);  break;
+    case 32:  fk_putp(att_bo);  break;
+ }
+ return(cur_attr = a);
 }
 
-void fk_colset(c)
-     int c;
+void fk_colset(int c)
 {
    int bg;
    if(cur_attr == c) return;
@@ -483,13 +555,13 @@ void fk_colset(c)
 #else
    if((c %= 16) >= col_num)
    {  fk_putp(att_bo);
-      e_putp(tparm(col_fg, c % col_num));
-      e_putp(tparm(col_bg, bg));
+      e_putp(tparm1(col_fg, c % col_num));
+      e_putp(tparm1(col_bg, bg));
    }
    else
    {  fk_putp(ratt_bo);
-      e_putp(tparm(col_fg, c));
-      e_putp(tparm(col_bg, bg));
+      e_putp(tparm1(col_fg, c));
+      e_putp(tparm1(col_bg, bg));
    }
 #endif
 }
@@ -504,7 +576,7 @@ int e_t_refresh()
       if(*(schirm + 2*MAXSCOL * i + 2 * j) != *(altschirm + 2*MAXSCOL * i + 2 * j) ||
          *(schirm + 2*MAXSCOL * i + 2 * j + 1) != *(altschirm + 2*MAXSCOL * i + 2 * j + 1) )
 #ifndef TERMCAP
-      {  if(cur_x != j || cur_y != i) e_putp(tparm(cur_rc, i, j));
+      {  if(cur_x != j || cur_y != i) e_putp(tparm2(cur_rc, i, j));
 #else
       {  if(cur_x != j || cur_y != i) e_putp(tgoto(cur_rc, j, i));
 #endif
@@ -527,59 +599,83 @@ int e_t_refresh()
 
 int e_t_sys_ini()
 {
-   e_refresh();
-   ioctl(0, TCGETA, &ttermio);
-   svflgs = fcntl( 0, F_GETFL, 0 );
-   e_endwin();
-   return(0);   
+ e_refresh();
+ tcgetattr(0, &ttermio);
+/* ioctl(0, TCGETA, &ttermio);*/
+ svflgs = fcntl( 0, F_GETFL, 0 );
+ e_endwin();
+ return(0);   
 }
 
 int e_t_sys_end()
 {
-   ioctl(0, TCSETA, &ttermio);
-   fcntl( 0, F_SETFL, svflgs );
-   e_abs_refr();
-   fk_locate(0, 0);
-   return(0);
+ tcsetattr(0, TCSADRAIN, &ttermio);
+/* ioctl(0, TCSETA, &ttermio);*/
+ fcntl( 0, F_SETFL, svflgs );
+ e_abs_refr();
+ fk_locate(0, 0);
+ return(0);
 }
 
 int e_t_kbhit()
 {
-   int ret;
-   char kbdflgs, c;
-   e_refresh();
-   kbdflgs = fcntl(0, F_GETFL, 0 );
-   fcntl( 0, F_SETFL, kbdflgs | O_NONBLOCK);
-   ret = read(0, &c, 1);
-   fcntl(0, F_SETFL, kbdflgs & ~O_NONBLOCK);
-   return(ret == 1 ? c : 0);
+ int ret;
+ char kbdflgs, c;
+
+ e_refresh();
+ kbdflgs = fcntl(0, F_GETFL, 0 );
+ fcntl(0, F_SETFL, kbdflgs | O_NONBLOCK);
+ ret = read(0, &c, 1);
+ fcntl(0, F_SETFL, kbdflgs & ~O_NONBLOCK);
+ return (ret == 1 ? c : 0);
 }
 
 extern char *key_key, *key_f[];
 
 int e_t_getch()
-{  int c, c2;
-   e_refresh();
-   if((c =fk_getch()) != ESC)
-   {  if(key_f[20] && c == *key_f[20]) return(DC);
-      else if(key_f[17] && c == *key_f[17]) return(ENTF);
-      else return(c);
-   }
-   else if((c = fk_getch()) == CR) return(ESC);
-   if(c == ESC)
-   {  if((c = fk_getch()) == ESC) return(ESC);
-      if((c2 = e_find_key(c, 1, 1))) return (c2);
-   }
-   if((c2 = e_find_key((char)c, 1, 0))) return (c2);
-   return(e_tast_sim(c));
+{
+ int c, c2, pshift, bk;
+
+ pshift = 0;
+ e_refresh();
+ if ((c = fk_getch()) != WPE_ESC)
+ {
+  if (key_f[20] && c == *key_f[20])
+   return(WPE_DC);
+  else if (key_f[17] && c == *key_f[17])
+   return(ENTF);
+  else if ( c == WPE_TAB )
+   { bk = bioskey(); 
+     if ( bk & 3) 
+         return (WPE_BTAB); 
+	    else 
+	 return (WPE_TAB); 
+    }
+  else
+   return(c);
+ }
+ else if ((c = fk_getch()) == WPE_CR)
+  return(WPE_ESC);
+ bk = bioskey();
+ if (bk & 3)
+  pshift = 512;
+ else if (bk & 4)
+  pshift = 514;
+ if (c == WPE_ESC)
+ {
+  if ((c = fk_getch()) == WPE_ESC)
+   return(WPE_ESC);
+  if ((c2 = e_find_key(c, 1, 1)))
+   return(c2 + pshift);
+ }
+ if ((c2 = e_find_key((char)c, 1, 0)))
+  return(c2 + pshift);
+ return(e_tast_sim(c + pshift));
 }
 
 char e_key_save[10];
 
-int e_find_key(c, j, sw)
-     int c;
-     int j;
-     int sw;
+int e_find_key(int c, int j, int sw)
 {
    int i, k;
    e_key_save[j] = c;
@@ -648,7 +744,7 @@ int e_find_key(c, j, sw)
 	       case 17:  return(ENTF);
 	       case 18:  return(ENDE);
 	       case 19:  return(BDO);
-	       case 20:  return(DC);
+	       case 20:  return(WPE_DC);
 	       case 21:  return(HELP);
 	       case 22:  return(ENDE);
 	       case 23:  return(SF1);
@@ -678,142 +774,130 @@ int e_find_key(c, j, sw)
    return(0);
 }
 
-int fk_t_locate(x, y)
-     int x;
-     int y;
+int fk_t_locate(int x, int y)
 {
-   if(col_num > 0) 
-   {   fk_colset(e_gt_col(cur_x, cur_y));
-       fputc(e_gt_char(cur_x, cur_y), stdout);
-   }
-   cur_x = x;
-   cur_y = y;
+ if (col_num > 0) 
+ {
+  fk_colset(e_gt_col(cur_x, cur_y));
+  fputc(e_gt_char(cur_x, cur_y), stdout);
+ }
+ cur_x = x;
+ cur_y = y;
 #ifndef TERMCAP
-   e_putp(tparm(cur_rc, y, x));
+ e_putp(tparm2(cur_rc, y, x));
 #else
-   e_putp(tgoto(cur_rc, x, y));
+ e_putp(tgoto(cur_rc, x, y));
 #endif
-   fflush(stdout);
-   return(y);
+ fflush(stdout);
+ return(y);
 }
 
-int fk_t_mouse(g)
-     int *g;
+int fk_t_mouse(int *g)
 {
-   return(0);
+ return(0);
 }
 
-#undef exit
-int e_exitm(s, n)
-     char *s;
-     int n;
+int e_t_switch_screen(int sw)
 {
-   e_endwin();
-   if(n != 0) printf("\n%s\n", s);
-   exit(n);
+ static int sav_sw = -1;
+
+ if (sw == sav_sw)
+  return(0);
+ sav_sw = sw;
+ if (sw && beg_scr) 
+ {
+  fflush(stdout);
+  if (sav_cur)
+   e_putp(sav_cur);
+  e_putp(beg_scr);
+ }
+ else if (!sw && swt_scr)
+ {
+  e_putp(swt_scr);
+  if (res_cur)
+   e_putp(res_cur);
+  fflush(stdout);
+ }
+ else
+  return(-1);
+ return(0);
 }
 
-int e_switch_screen(sw)
-     int sw;
+int e_t_deb_out(FENSTER *f)
 {
-   static int sav_sw = -1;
-   if((e_we_sw & 1) || sw == sav_sw) return(0);
-   sav_sw = sw;
-   if(sw && beg_scr) 
-   {  fflush(stdout);
-      if(sav_cur) e_putp(sav_cur);
-      e_putp(beg_scr);
-/*      fflush(stdout);    */
-   }
-   else if(!sw && swt_scr)
-   {  e_putp(swt_scr);
-      if(res_cur) e_putp(res_cur);
-      fflush(stdout);
-   }
-   else return(-1);
-   return(0);
+ if (!swt_scr || !beg_scr)
+  return(e_error("Your terminal don\'t use begin/end cup", 0, f->fb));
+ e_d_switch_out(1);
+ getchar();
+ e_d_switch_out(0);
+ return(0);
 }
 
-int e_deb_out(f)
-     FENSTER *f;
+int e_d_switch_screen(int sw)
 {
-   if(!swt_scr || !beg_scr)
-      return(e_error("Your terminal don\'t use begin/end cup", 0, f->fb));
-   e_d_switch_out(1);
-   getchar();
-   e_d_switch_out(0);
-   return(0);
-}
-
-int e_d_switch_screen(sw)
-     int sw;
-{
-   if(e_we_sw & 1) return(0);
 #ifdef DEBUGGER
-   if(!sw) e_g_sys_ini();
-   else e_g_sys_end();
+ if (!sw)
+  e_g_sys_ini();
+ else
+  e_g_sys_end();
 #endif
-   return(e_switch_screen(sw));
+ return(e_switch_screen(sw));
 }
 
-int e_d_switch_out(sw)
-     int sw;
+int e_t_d_switch_out(int sw)
 {
-   int i, j;
-   static int save_sw = 32000;
-   if(e_we_sw & 1) return(sw);
-   if(save_sw == sw) return(0);
-   save_sw = sw;
-/*
-   if(e_d_out == NULL)
-   {  e_d_out = MALLOC(MAXOUT + 1);
-      e_d_out_cur = 0;
-   }
-*/
-   if(sw && e_d_switch_screen(0))
+ int i, j;
+ static int save_sw = 32000;
+
+ if (save_sw == sw)
+  return(0);
+ save_sw = sw;
+ if (sw && e_d_switch_screen(0))
+ {
 #ifndef TERMCAP
-   {  e_putp(tparm(cur_rc, 0, 0));
+  e_putp(tparm2(cur_rc, 0, 0));
 #else
-   {  e_putp(tgoto(cur_rc, 0, 0));
+  e_putp(tgoto(cur_rc, 0, 0));
 #endif
-      e_putp(att_no);
-      for(i = 0; i < MAXSLNS; i++)
-	 for(j = 0; j < MAXSCOL; j++)
-	 e_d_putchar(' ');
+  e_putp(att_no);
+  for(i = 0; i < MAXSLNS; i++)
+   for (j = 0; j < MAXSCOL; j++)
+    e_d_putchar(' ');
 #ifndef TERMCAP
-      e_putp(tparm(cur_rc, 0, 0));
+  e_putp(tparm2(cur_rc, 0, 0));
 #else
-      e_putp(tgoto(cur_rc, 0, 0));
+  e_putp(tgoto(cur_rc, 0, 0));
 #endif
-/*
-      for(i = 0; i < e_d_out_cur; i++)
-      {  if(e_d_out[i] == '\n')  e_d_putchar(CR);
-	 e_d_putchar(e_d_out[i] & 255);
-      }
-*/
-      fflush(stdout);
-   }
-   else if(!sw)
-   {  e_d_switch_screen(1);
-      e_abs_refr();
-      e_refresh();
-   }
-   return(sw);
+  fflush(stdout);
+ }
+ else if (!sw)
+ {
+  e_d_switch_screen(1);
+  e_abs_refr();
+  e_refresh();
+ }
+ return(sw);
+}
+
+void e_exitm(char *s, int n)
+{
+ e_endwin();
+ if (n != 0)
+  printf("\n%s\n", s);
+ exit(n);
 }
-#endif  /*  is not DJGPP  */
 
 int e_s_sys_ini()
 {
-   e_sys_ini();
-   return(e_switch_screen(0));
+ e_sys_ini();
+ return(e_switch_screen(0));
 }
 
 int e_s_sys_end()
 {
-   e_switch_screen(1);
-   return(e_sys_end());
+ e_switch_screen(1);
+ return(e_sys_end());
 }
 
 #endif  /*  Is UNIX       */
 
-
